@@ -3,8 +3,9 @@ import numpy as np
 from sklearn.model_selection import cross_val_predict
 from sklearn.base import clone
 from sklearn.preprocessing import LabelEncoder
+from sklearn.model_selection import KFold
+from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
 
-import warnings
 from joblib import Parallel, delayed
 
 
@@ -91,12 +92,17 @@ def _dml_cv_predict(estimator, x, y, smpls=None,
     if not manual_cv_predict:
         if est_params is None:
             # if there are no parameters set we redirect to the standard method
-            return cross_val_predict(clone(estimator), x, y, cv=smpls, n_jobs=n_jobs, method=method)
-        elif isinstance(est_params, dict):
+            preds = cross_val_predict(clone(estimator), x, y, cv=smpls, n_jobs=n_jobs, method=method)
+        else:
+            assert isinstance(est_params, dict)
             # if no fold-specific parameters we redirect to the standard method
             # warnings.warn("Using the same (hyper-)parameters for all folds")
-            return cross_val_predict(clone(estimator).set_params(**est_params), x, y, cv=smpls, n_jobs=n_jobs,
-                                     method=method)
+            preds = cross_val_predict(clone(estimator).set_params(**est_params), x, y, cv=smpls, n_jobs=n_jobs,
+                                      method=method)
+        if method == 'predict_proba':
+            return preds[:, 1]
+        else:
+            return preds
     else:
         if not smpls_is_partition:
             assert not fold_specific_target, 'combination of fold-specific y and no cross-fitting not implemented yet'
@@ -136,14 +142,12 @@ def _dml_cv_predict(estimator, x, y, smpls=None,
                                      for idx, (train_index, test_index) in enumerate(smpls))
 
         preds = np.full(n_obs, np.nan)
-        if method == 'predict_proba':
-            preds = np.full((n_obs, 2), np.nan)
         train_preds = list()
         for idx, (train_index, test_index) in enumerate(smpls):
             assert idx == fitted_models[idx][1]
             pred_fun = getattr(fitted_models[idx][0], method)
             if method == 'predict_proba':
-                preds[test_index, :] = pred_fun(x[test_index, :])
+                preds[test_index] = pred_fun(x[test_index, :])[:, 1]
             else:
                 preds[test_index] = pred_fun(x[test_index, :])
 
@@ -156,6 +160,27 @@ def _dml_cv_predict(estimator, x, y, smpls=None,
             return preds
 
 
+def _dml_tune(y, x, train_inds,
+              learner, param_grid, scoring_method,
+              n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search):
+    tune_res = list()
+    for train_index in train_inds:
+        tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
+        if search_mode == 'grid_search':
+            g_grid_search = GridSearchCV(learner, param_grid,
+                                         scoring=scoring_method,
+                                         cv=tune_resampling, n_jobs=n_jobs_cv)
+        else:
+            assert search_mode == 'randomized_search'
+            g_grid_search = RandomizedSearchCV(learner, param_grid,
+                                               scoring=scoring_method,
+                                               cv=tune_resampling, n_jobs=n_jobs_cv,
+                                               n_iter=n_iter_randomized_search)
+        tune_res.append(g_grid_search.fit(x[train_index, :], y[train_index]))
+
+    return tune_res
+
+
 def _draw_weights(method, n_rep_boot, n_obs):
     if method == 'Bayes':
         weights = np.random.exponential(scale=1.0, size=(n_rep_boot, n_obs)) - 1.
