@@ -1,16 +1,13 @@
 import numpy as np
 from sklearn.utils import check_X_y
 from sklearn.utils.multiclass import type_of_target
-from sklearn.model_selection import KFold
-from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
 
 from .double_ml import DoubleML
-from ._helper import _dml_cv_predict, _get_cond_smpls
+from ._helper import _dml_cv_predict, _get_cond_smpls, _dml_tune
 
 
 class DoubleMLIIVM(DoubleML):
-    """
-    Double machine learning for interactive IV regression models
+    """Double machine learning for interactive IV regression models
 
     Parameters
     ----------
@@ -18,16 +15,16 @@ class DoubleMLIIVM(DoubleML):
         The :class:`DoubleMLData` object providing the data and specifying the variables for the causal model.
 
     ml_g : estimator implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestRegressor`)
-        for the nuisance function :math:`g_0(Z,X) = E[Y|X,Z]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestRegressor`) for the nuisance function :math:`g_0(Z,X) = E[Y|X,Z]`.
 
     ml_m : classifier implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestClassifier`)
-        for the nuisance function :math:`m_0(X) = E[Z|X]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestClassifier`) for the nuisance function :math:`m_0(X) = E[Z|X]`.
 
     ml_r : classifier implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestClassifier`)
-        for the nuisance function :math:`r_0(Z,X) = E[D|X,Z]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestClassifier`) for the nuisance function :math:`r_0(Z,X) = E[D|X,Z]`.
 
     n_folds : int
         Number of folds.
@@ -39,7 +36,8 @@ class DoubleMLIIVM(DoubleML):
 
     score : str or callable
         A str (``'LATE'`` is the only choice) specifying the score function
-        or a callable object / function with signature ``psi_a, psi_b = score(y, z, d, g_hat0, g_hat1, m_hat, r_hat0, r_hat1, smpls)``.
+        or a callable object / function with signature
+        ``psi_a, psi_b = score(y, z, d, g_hat0, g_hat1, m_hat, r_hat0, r_hat1, smpls)``.
         Default is ``'LATE'``.
 
     dml_procedure : str
@@ -81,7 +79,33 @@ class DoubleMLIIVM(DoubleML):
 
     Notes
     -----
-    .. include:: ../../shared/models/iivm.rst
+    **Interactive IV regression (IIVM)** models take the form
+
+    .. math::
+
+        Y = \\ell_0(D, X) + \\zeta, & &\\mathbb{E}(\\zeta | Z, X) = 0,
+
+        Z = m_0(X) + V, & &\\mathbb{E}(V | X) = 0,
+
+    where the treatment variable is binary, :math:`D \\in \\lbrace 0,1 \\rbrace`
+    and the instrument is binary, :math:`Z \\in \\lbrace 0,1 \\rbrace`.
+    Consider the functions :math:`g_0`, :math:`r_0` and :math:`m_0`, where :math:`g_0` maps the support of :math:`(Z,X)` to
+    :math:`\\mathbb{R}` and :math:`r_0` and :math:`m_0` respectively map the support of :math:`(Z,X)` and :math:`X` to
+    :math:`(\\varepsilon, 1-\\varepsilon)` for some :math:`\\varepsilon \\in (0, 1/2)`, such that
+
+    .. math::
+
+        Y = g_0(Z, X) + \\nu, & &\\mathbb{E}(\\nu| Z, X) = 0,
+
+        D = r_0(Z, X) + U, & &\\mathbb{E}(U | Z, X) = 0,
+
+        Z = m_0(X) + V, & &\\mathbb{E}(V | X) = 0.
+
+    The target parameter of interest in this model is the local average treatment effect (LATE),
+
+    .. math::
+
+        \\theta_0 = \\frac{\\mathbb{E}[g_0(1, X)] - \\mathbb{E}[g_0(0,X)]}{\\mathbb{E}[r_0(1, X)] - \\mathbb{E}[r_0(0,X)]}.
     """
     def __init__(self,
                  obj_dml_data,
@@ -103,15 +127,17 @@ class DoubleMLIIVM(DoubleML):
                          dml_procedure,
                          draw_sample_splitting,
                          apply_cross_fitting)
-        self._learner = {'ml_g': self._check_learner(ml_g, 'ml_g'),
-                         'ml_m': self._check_learner(ml_m, 'ml_m', classifier=True),
-                         'ml_r':  self._check_learner(ml_r, 'ml_r', classifier=True)}
+        _ = self._check_learner(ml_g, 'ml_g', regressor=True, classifier=False)
+        _ = self._check_learner(ml_m, 'ml_m', regressor=False, classifier=True)
+        _ = self._check_learner(ml_r, 'ml_r', regressor=False, classifier=True)
+        self._learner = {'ml_g': ml_g, 'ml_m': ml_m, 'ml_r': ml_r}
+        self._predict_method = {'ml_g': 'predict', 'ml_m': 'predict_proba', 'ml_r': 'predict_proba'}
         self._initialize_ml_nuisance_params()
 
         valid_trimming_rule = ['truncate']
         if trimming_rule not in valid_trimming_rule:
-            raise ValueError('invalid trimming_rule ' + trimming_rule +
-                             '\n valid trimming_rule ' + ' or '.join(valid_trimming_rule))
+            raise ValueError('Invalid trimming_rule ' + trimming_rule + '. ' +
+                             'Valid trimming_rule ' + ' or '.join(valid_trimming_rule) + '.')
         self.trimming_rule = trimming_rule
         self.trimming_threshold = trimming_threshold
 
@@ -124,33 +150,33 @@ class DoubleMLIIVM(DoubleML):
         if isinstance(score, str):
             valid_score = ['LATE']
             if score not in valid_score:
-                raise ValueError('invalid score ' + score +
-                                 '\n valid score ' + ' or '.join(valid_score))
+                raise ValueError('Invalid score ' + score + '. '
+                                 'Valid score ' + ' or '.join(valid_score) + '.')
         else:
             if not callable(score):
-                raise ValueError('score should be either a string or a callable.'
-                                 ' %r was passed' % score)
+                raise TypeError('score should be either a string or a callable. '
+                                '%r was passed.' % score)
         return score
 
     def _check_data(self, obj_dml_data):
         one_treat = (obj_dml_data.n_treat == 1)
         binary_treat = (type_of_target(obj_dml_data.d) == 'binary')
-        zero_one_treat = np.any((np.power(obj_dml_data.d, 2) - obj_dml_data.d) == 0)
+        zero_one_treat = np.all((np.power(obj_dml_data.d, 2) - obj_dml_data.d) == 0)
         if not(one_treat & binary_treat & zero_one_treat):
-            raise ValueError('Incompatible data.\n'
+            raise ValueError('Incompatible data. '
                              'To fit an IIVM model with DML '
                              'exactly one binary variable with values 0 and 1 '
                              'needs to be specified as treatment variable.')
         one_instr = (obj_dml_data.n_instr == 1)
         binary_instr = (type_of_target(obj_dml_data.z) == 'binary')
-        zero_one_instr = np.any((np.power(obj_dml_data.z, 2) - obj_dml_data.z) == 0)
+        zero_one_instr = np.all((np.power(obj_dml_data.z, 2) - obj_dml_data.z) == 0)
         if not(one_instr & binary_instr & zero_one_instr):
-            raise ValueError('Incompatible data.\n'
+            raise ValueError('Incompatible data. '
                              'To fit an IIVM model with DML '
                              'exactly one binary variable with values 0 and 1 '
                              'needs to be specified as instrumental variable.')
         return
-    
+
     def _ml_nuisance_and_score_elements(self, smpls, n_jobs_cv):
         x, y = check_X_y(self._dml_data.x, self._dml_data.y)
         x, z = check_X_y(x, np.ravel(self._dml_data.z))
@@ -158,22 +184,22 @@ class DoubleMLIIVM(DoubleML):
 
         # get train indices for z == 0 and z == 1
         smpls_z0, smpls_z1 = _get_cond_smpls(smpls, z)
-        
+
         # nuisance g
         g_hat0 = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls_z0, n_jobs=n_jobs_cv,
-                                 est_params=self._get_params('ml_g0'))
+                                 est_params=self._get_params('ml_g0'), method=self._predict_method['ml_g'])
         g_hat1 = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls_z1, n_jobs=n_jobs_cv,
-                                 est_params=self._get_params('ml_g1'))
-        
+                                 est_params=self._get_params('ml_g1'), method=self._predict_method['ml_g'])
+
         # nuisance m
-        m_hat = _dml_cv_predict(self._learner['ml_m'], x, z, smpls=smpls, method='predict_proba', n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_m'))[:, 1]
-        
+        m_hat = _dml_cv_predict(self._learner['ml_m'], x, z, smpls=smpls, n_jobs=n_jobs_cv,
+                                est_params=self._get_params('ml_m'), method=self._predict_method['ml_m'])
+
         # nuisance r
-        r_hat0 = _dml_cv_predict(self._learner['ml_r'], x, d, smpls=smpls_z0, method='predict_proba', n_jobs=n_jobs_cv,
-                                 est_params=self._get_params('ml_r0'))[:, 1]
-        r_hat1 = _dml_cv_predict(self._learner['ml_r'], x, d, smpls=smpls_z1, method='predict_proba', n_jobs=n_jobs_cv,
-                                 est_params=self._get_params('ml_r1'))[:, 1]
+        r_hat0 = _dml_cv_predict(self._learner['ml_r'], x, d, smpls=smpls_z0, n_jobs=n_jobs_cv,
+                                 est_params=self._get_params('ml_r0'), method=self._predict_method['ml_r'])
+        r_hat1 = _dml_cv_predict(self._learner['ml_r'], x, d, smpls=smpls_z1, n_jobs=n_jobs_cv,
+                                 est_params=self._get_params('ml_r1'), method=self._predict_method['ml_r'])
 
         psi_a, psi_b = self._score_elements(y, z, d, g_hat0, g_hat1, m_hat, r_hat0, r_hat1, smpls)
 
@@ -218,84 +244,25 @@ class DoubleMLIIVM(DoubleML):
                                'ml_m': None,
                                'ml_r': None}
 
-        g0_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            g0_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                g0_grid_search = GridSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                              scoring=scoring_methods['ml_g'],
-                                              cv=g0_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                g0_grid_search = RandomizedSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                    scoring=scoring_methods['ml_g'],
-                                                    cv=g0_tune_resampling, n_jobs=n_jobs_cv,
-                                                    n_iter=n_iter_randomized_search)
-            train_index_z0 = smpls_z0[idx][0]
-            g0_tune_res.append(g0_grid_search.fit(x[train_index_z0, :], y[train_index_z0]))
+        train_inds = [train_index for (train_index, _) in smpls]
+        train_inds_z0 = [train_index for (train_index, _) in smpls_z0]
+        train_inds_z1 = [train_index for (train_index, _) in smpls_z1]
 
-        g1_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            g1_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                g1_grid_search = GridSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                              scoring=scoring_methods['ml_g'],
-                                              cv=g1_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                g1_grid_search = RandomizedSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                    scoring=scoring_methods['ml_g'],
-                                                    cv=g1_tune_resampling, n_jobs=n_jobs_cv,
-                                                    n_iter=n_iter_randomized_search)
-            train_index_z1 = smpls_z1[idx][0]
-            g1_tune_res.append(g1_grid_search.fit(x[train_index_z1, :], y[train_index_z1]))
-
-        m_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            m_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                m_grid_search = GridSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                             scoring=scoring_methods['ml_m'],
-                                             cv=m_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                m_grid_search = RandomizedSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                   scoring=scoring_methods['ml_m'],
-                                                   cv=m_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            m_tune_res.append(m_grid_search.fit(x[train_index, :], z[train_index]))
-
-        r0_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            r0_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                r0_grid_search = GridSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                              scoring=scoring_methods['ml_r'],
-                                              cv=r0_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                r0_grid_search = RandomizedSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                                    scoring=scoring_methods['ml_r'],
-                                                    cv=r0_tune_resampling, n_jobs=n_jobs_cv,
-                                                    n_iter=n_iter_randomized_search)
-            train_index_z0 = smpls_z0[idx][0]
-            r0_tune_res.append(r0_grid_search.fit(x[train_index_z0, :], d[train_index_z0]))
-
-        r1_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            r1_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                r1_grid_search = GridSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                              scoring=scoring_methods['ml_r'],
-                                              cv=r1_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                r1_grid_search = RandomizedSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                                    scoring=scoring_methods['ml_r'],
-                                                    cv=r1_tune_resampling, n_jobs=n_jobs_cv,
-                                                    n_iter=n_iter_randomized_search)
-            train_index_z1 = smpls_z1[idx][0]
-            r1_tune_res.append(r1_grid_search.fit(x[train_index_z1, :], d[train_index_z1]))
+        g0_tune_res = _dml_tune(y, x, train_inds_z0,
+                                self._learner['ml_g'], param_grids['ml_g'], scoring_methods['ml_g'],
+                                n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
+        g1_tune_res = _dml_tune(y, x, train_inds_z1,
+                                self._learner['ml_g'], param_grids['ml_g'], scoring_methods['ml_g'],
+                                n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
+        m_tune_res = _dml_tune(z, x, train_inds,
+                               self._learner['ml_m'], param_grids['ml_m'], scoring_methods['ml_m'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
+        r0_tune_res = _dml_tune(d, x, train_inds_z0,
+                                self._learner['ml_r'], param_grids['ml_r'], scoring_methods['ml_r'],
+                                n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
+        r1_tune_res = _dml_tune(d, x, train_inds_z1,
+                                self._learner['ml_r'], param_grids['ml_r'], scoring_methods['ml_r'],
+                                n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
         g0_best_params = [xx.best_params_ for xx in g0_tune_res]
         g1_best_params = [xx.best_params_ for xx in g1_tune_res]
