@@ -1,16 +1,13 @@
 import numpy as np
 from sklearn.utils import check_X_y
 from sklearn.utils.multiclass import type_of_target
-from sklearn.model_selection import KFold
-from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
 
 from .double_ml import DoubleML
-from ._helper import _dml_cv_predict, _get_cond_smpls
+from ._helper import _dml_cv_predict, _get_cond_smpls, _dml_tune
 
 
 class DoubleMLIRM(DoubleML):
-    """
-    Double machine learning for interactive regression models
+    """Double machine learning for interactive regression models
 
     Parameters
     ----------
@@ -18,12 +15,12 @@ class DoubleMLIRM(DoubleML):
         The :class:`DoubleMLData` object providing the data and specifying the variables for the causal model.
 
     ml_g : estimator implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestRegressor`)
-        for the nuisance function :math:`g_0(D,X) = E[Y|X,D]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestRegressor`) for the nuisance function :math:`g_0(D,X) = E[Y|X,D]`.
 
     ml_m : classifier implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestClassifier`)
-        for the nuisance function :math:`m_0(X) = E[D|X]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestClassifier`) for the nuisance function :math:`m_0(X) = E[D|X]`.
 
     n_folds : int
         Number of folds.
@@ -76,7 +73,27 @@ class DoubleMLIRM(DoubleML):
 
     Notes
     -----
-    .. include:: ../../shared/models/irm.rst
+    **Interactive regression (IRM)** models take the form
+
+    .. math::
+
+        Y = g_0(D, X) + U, & &\\mathbb{E}(U | X, D) = 0,
+
+        D = m_0(X) + V, & &\\mathbb{E}(V | X) = 0,
+
+    where the treatment variable is binary, :math:`D \\in \\lbrace 0,1 \\rbrace`.
+    We consider estimation of the average treatment effects when treatment effects are fully heterogeneous.
+    Target parameters of interest in this model are the average treatment effect (ATE),
+
+    .. math::
+
+        \\theta_0 = \\mathbb{E}[g_0(1, X) - g_0(0,X)]
+
+    and the average treatment effect of the treated (ATTE),
+
+    .. math::
+
+        \\theta_0 = \\mathbb{E}[g_0(1, X) - g_0(0,X) | D=1].
     """
     def __init__(self,
                  obj_dml_data,
@@ -97,14 +114,16 @@ class DoubleMLIRM(DoubleML):
                          dml_procedure,
                          draw_sample_splitting,
                          apply_cross_fitting)
-        self._learner = {'ml_g': self._check_learner(ml_g, 'ml_g'),
-                         'ml_m': self._check_learner(ml_m, 'ml_m', classifier=True)}
+        _ = self._check_learner(ml_g, 'ml_g', regressor=True, classifier=False)
+        _ = self._check_learner(ml_m, 'ml_m', regressor=False, classifier=True)
+        self._learner = {'ml_g': ml_g, 'ml_m': ml_m}
+        self._predict_method = {'ml_g': 'predict', 'ml_m': 'predict_proba'}
         self._initialize_ml_nuisance_params()
 
         valid_trimming_rule = ['truncate']
         if trimming_rule not in valid_trimming_rule:
-            raise ValueError('invalid trimming_rule ' + trimming_rule +
-                             '\n valid trimming_rule ' + ' or '.join(valid_trimming_rule))
+            raise ValueError('Invalid trimming_rule ' + trimming_rule + '. ' +
+                             'Valid trimming_rule ' + ' or '.join(valid_trimming_rule) + '.')
         self.trimming_rule = trimming_rule
         self.trimming_threshold = trimming_threshold
 
@@ -117,30 +136,30 @@ class DoubleMLIRM(DoubleML):
         if isinstance(score, str):
             valid_score = ['ATE', 'ATTE']
             if score not in valid_score:
-                raise ValueError('invalid score ' + score +
-                                 '\n valid score ' + ' or '.join(valid_score))
+                raise ValueError('Invalid score ' + score + '. ' +
+                                 'Valid score ' + ' or '.join(valid_score) + '.')
         else:
             if not callable(score):
-                raise ValueError('score should be either a string or a callable.'
-                                 ' %r was passed' % score)
+                raise TypeError('score should be either a string or a callable. '
+                                '%r was passed.' % score)
         return score
 
     def _check_data(self, obj_dml_data):
         if obj_dml_data.z_cols is not None:
-            raise ValueError('Incompatible data.\n'
+            raise ValueError('Incompatible data. ' +
                              ' and '.join(obj_dml_data.z_cols) +
-                             'have been set as instrumental variable(s).\n'
-                             'To fit an interactiv IV regression model use DoubleMLIIVM instead of DoubleMLIRM.')
+                             ' have been set as instrumental variable(s). '
+                             'To fit an interactive IV regression model use DoubleMLIIVM instead of DoubleMLIRM.')
         one_treat = (obj_dml_data.n_treat == 1)
         binary_treat = (type_of_target(obj_dml_data.d) == 'binary')
-        zero_one_treat = np.any((np.power(obj_dml_data.d, 2) - obj_dml_data.d) == 0)
+        zero_one_treat = np.all((np.power(obj_dml_data.d, 2) - obj_dml_data.d) == 0)
         if not(one_treat & binary_treat & zero_one_treat):
-            raise ValueError('Incompatible data.\n'
-                             'To fit an IIVM model with DML '
+            raise ValueError('Incompatible data. '
+                             'To fit an IRM model with DML '
                              'exactly one binary variable with values 0 and 1 '
                              'needs to be specified as treatment variable.')
         return
-    
+
     def _ml_nuisance_and_score_elements(self, smpls, n_jobs_cv):
         x, y = check_X_y(self._dml_data.x, self._dml_data.y)
         x, d = check_X_y(x, self._dml_data.d)
@@ -149,15 +168,15 @@ class DoubleMLIRM(DoubleML):
 
         # nuisance g
         g_hat0 = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls_d0, n_jobs=n_jobs_cv,
-                                 est_params=self._get_params('ml_g0'))
+                                 est_params=self._get_params('ml_g0'), method=self._predict_method['ml_g'])
         g_hat1 = None
         if (self.score == 'ATE') | callable(self.score):
             g_hat1 = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls_d1, n_jobs=n_jobs_cv,
-                                     est_params=self._get_params('ml_g1'))
-        
+                                     est_params=self._get_params('ml_g1'), method=self._predict_method['ml_g'])
+
         # nuisance m
-        m_hat = _dml_cv_predict(self._learner['ml_m'], x, d, smpls=smpls, method='predict_proba', n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_m'))[:, 1]
+        m_hat = _dml_cv_predict(self._learner['ml_m'], x, d, smpls=smpls, n_jobs=n_jobs_cv,
+                                est_params=self._get_params('ml_m'), method=self._predict_method['ml_m'])
 
         psi_a, psi_b = self._score_elements(y, d, g_hat0, g_hat1, m_hat, smpls)
 
@@ -210,53 +229,21 @@ class DoubleMLIRM(DoubleML):
             scoring_methods = {'ml_g': None,
                                'ml_m': None}
 
-        g0_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            g0_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                g0_grid_search = GridSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                              scoring=scoring_methods['ml_g'],
-                                              cv=g0_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                g0_grid_search = RandomizedSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                    scoring=scoring_methods['ml_g'],
-                                                    cv=g0_tune_resampling, n_jobs=n_jobs_cv,
-                                                    n_iter=n_iter_randomized_search)
-            train_index_d0 = smpls_d0[idx][0]
-            g0_tune_res.append(g0_grid_search.fit(x[train_index_d0, :], y[train_index_d0]))
-
+        train_inds = [train_index for (train_index, _) in smpls]
+        train_inds_d0 = [train_index for (train_index, _) in smpls_d0]
+        train_inds_d1 = [train_index for (train_index, _) in smpls_d1]
+        g0_tune_res = _dml_tune(y, x, train_inds_d0,
+                                self._learner['ml_g'], param_grids['ml_g'], scoring_methods['ml_g'],
+                                n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
         g1_tune_res = list()
         if self.score == 'ATE':
-            for idx, (train_index, test_index) in enumerate(smpls):
-                g1_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-                if search_mode == 'grid_search':
-                    g1_grid_search = GridSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                  scoring=scoring_methods['ml_g'],
-                                                  cv=g1_tune_resampling, n_jobs=n_jobs_cv)
-                else:
-                    assert search_mode == 'randomized_search'
-                    g1_grid_search = RandomizedSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                        scoring=scoring_methods['ml_g'],
-                                                        cv=g1_tune_resampling, n_jobs=n_jobs_cv,
-                                                        n_iter=n_iter_randomized_search)
-                train_index_d1 = smpls_d1[idx][0]
-                g1_tune_res.append(g1_grid_search.fit(x[train_index_d1, :], y[train_index_d1]))
+            g1_tune_res = _dml_tune(y, x, train_inds_d1,
+                                    self._learner['ml_g'], param_grids['ml_g'], scoring_methods['ml_g'],
+                                    n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
-        m_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            m_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                m_grid_search = GridSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                             scoring=scoring_methods['ml_m'],
-                                             cv=m_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                m_grid_search = RandomizedSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                   scoring=scoring_methods['ml_m'],
-                                                   cv=m_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            m_tune_res.append(m_grid_search.fit(x[train_index, :], d[train_index]))
+        m_tune_res = _dml_tune(d, x, train_inds,
+                               self._learner['ml_m'], param_grids['ml_m'], scoring_methods['ml_m'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
         g0_best_params = [xx.best_params_ for xx in g0_tune_res]
         m_best_params = [xx.best_params_ for xx in m_tune_res]
