@@ -6,12 +6,11 @@ from sklearn.linear_model import LinearRegression
 from sklearn.dummy import DummyRegressor
 
 from .double_ml import DoubleML
-from ._helper import _dml_cv_predict
+from ._helper import _dml_cv_predict, _dml_tune
 
 
 class DoubleMLPLIV(DoubleML):
-    """
-    Double machine learning for partially linear IV regression models
+    """Double machine learning for partially linear IV regression models
 
     Parameters
     ----------
@@ -19,16 +18,16 @@ class DoubleMLPLIV(DoubleML):
         The :class:`DoubleMLData` object providing the data and specifying the variables for the causal model.
 
     ml_g : estimator implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestRegressor`)
-        for the nuisance function :math:`g_0(X) = E[Y|X]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestRegressor`) for the nuisance function :math:`g_0(X) = E[Y|X]`.
 
     ml_m : estimator implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestRegressor`)
-        for the nuisance function :math:`m_0(X) = E[Z|X]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestRegressor`) for the nuisance function :math:`m_0(X) = E[Z|X]`.
 
     ml_r : estimator implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestRegressor`)
-        for the nuisance function :math:`r_0(X) = E[D|X]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestRegressor`) for the nuisance function :math:`r_0(X) = E[D|X]`.
 
     n_folds : int
         Number of folds.
@@ -76,7 +75,18 @@ class DoubleMLPLIV(DoubleML):
 
     Notes
     -----
-    .. include:: ../../shared/models/pliv.rst
+    **Partially linear IV regression (PLIV)** models take the form
+
+    .. math::
+
+        Y - D \\theta_0 =  g_0(X) + \\zeta, & &\\mathbb{E}(\\zeta | Z, X) = 0,
+
+        Z = m_0(X) + V, & &\\mathbb{E}(V | X) = 0.
+
+    where :math:`Y` is the outcome variable, :math:`D` is the policy variable of interest and :math:`Z`
+    denotes one or multiple instrumental variables. The high-dimensional vector
+    :math:`X = (X_1, \\ldots, X_p)` consists of other confounding covariates, and :math:`\\zeta` and
+    :math:`V` are stochastic errors.
     """
     def __init__(self,
                  obj_dml_data,
@@ -98,9 +108,11 @@ class DoubleMLPLIV(DoubleML):
                          apply_cross_fitting)
         self.partialX = True
         self.partialZ = False
-        self._learner = {'ml_g': self._check_learner(ml_g, 'ml_g'),
-                         'ml_m': self._check_learner(ml_m, 'ml_m'),
-                         'ml_r': self._check_learner(ml_r, 'ml_r')}
+        _ = self._check_learner(ml_g, 'ml_g', regressor=True, classifier=False)
+        _ = self._check_learner(ml_m, 'ml_m', regressor=True, classifier=False)
+        _ = self._check_learner(ml_r, 'ml_r', regressor=True, classifier=False)
+        self._learner = {'ml_g': ml_g, 'ml_m': ml_m, 'ml_r': ml_r}
+        self._predict_method = {'ml_g': 'predict', 'ml_m': 'predict', 'ml_r': 'predict'}
         self._initialize_ml_nuisance_params()
 
     @classmethod
@@ -127,9 +139,11 @@ class DoubleMLPLIV(DoubleML):
                   apply_cross_fitting)
         obj.partialX = True
         obj.partialZ = False
-        obj._learner = {'ml_g': obj._check_learner(ml_g, 'ml_g'),
-                        'ml_m': obj._check_learner(ml_m, 'ml_m'),
-                        'ml_r': obj._check_learner(ml_r, 'ml_r')}
+        _ = obj._check_learner(ml_g, 'ml_g', regressor=True, classifier=False)
+        _ = obj._check_learner(ml_m, 'ml_m', regressor=True, classifier=False)
+        _ = obj._check_learner(ml_r, 'ml_r', regressor=True, classifier=False)
+        obj._learner = {'ml_g': ml_g, 'ml_m': ml_m, 'ml_r': ml_r}
+        obj._predict_method = {'ml_g': 'predict', 'ml_m': 'predict', 'ml_r': 'predict'}
         obj._initialize_ml_nuisance_params()
         return obj
 
@@ -156,7 +170,9 @@ class DoubleMLPLIV(DoubleML):
                   apply_cross_fitting)
         obj.partialX = False
         obj.partialZ = True
-        obj._learner = {'ml_r': obj._check_learner(ml_r, 'ml_r')}
+        _ = obj._check_learner(ml_r, 'ml_r', regressor=True, classifier=False)
+        obj._learner = {'ml_r': ml_r}
+        obj._predict_method = {'ml_r': 'predict'}
         obj._initialize_ml_nuisance_params()
         return obj
 
@@ -184,9 +200,11 @@ class DoubleMLPLIV(DoubleML):
                   apply_cross_fitting)
         obj.partialX = True
         obj.partialZ = True
-        obj._learner = {'ml_g': obj._check_learner(ml_g, 'ml_g'),
-                        'ml_m': obj._check_learner(ml_m, 'ml_m'),
-                        'ml_r': obj._check_learner(ml_r, 'ml_r')}
+        _ = obj._check_learner(ml_g, 'ml_g', regressor=True, classifier=False)
+        _ = obj._check_learner(ml_m, 'ml_m', regressor=True, classifier=False)
+        _ = obj._check_learner(ml_r, 'ml_r', regressor=True, classifier=False)
+        obj._learner = {'ml_g': ml_g, 'ml_m': ml_m, 'ml_r': ml_r}
+        obj._predict_method = {'ml_g': 'predict', 'ml_m': 'predict', 'ml_r': 'predict'}
         obj._initialize_ml_nuisance_params()
         return obj
 
@@ -212,12 +230,12 @@ class DoubleMLPLIV(DoubleML):
             # however in the following paragraph it is explained that one might
             # still need to estimate the partialling out type first
             if score not in valid_score:
-                raise ValueError('invalid score ' + score +
-                                 '\n valid score ' + 'partialling out')
+                raise ValueError('Invalid score ' + score + '. ' +
+                                 'Valid score ' + 'partialling out.')
         else:
             if not callable(score):
-                raise ValueError('score should be either a string or a callable.'
-                                 ' %r was passed' % score)
+                raise TypeError('score should be either a string or a callable. '
+                                '%r was passed.' % score)
         return score
 
     def _check_data(self, obj_dml_data):
@@ -252,17 +270,17 @@ class DoubleMLPLIV(DoubleML):
     def _ml_nuisance_and_score_elements_partial_x(self, smpls, n_jobs_cv):
         x, y = check_X_y(self._dml_data.x, self._dml_data.y)
         x, d = check_X_y(x, self._dml_data.d)
-        
+
         # nuisance g
         g_hat = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls, n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_g'))
-        
+                                est_params=self._get_params('ml_g'), method=self._predict_method['ml_g'])
+
         # nuisance m
         if self._dml_data.n_instr == 1:
             # one instrument: just identified
             x, z = check_X_y(x, np.ravel(self._dml_data.z))
             m_hat = _dml_cv_predict(self._learner['ml_m'], x, z, smpls=smpls, n_jobs=n_jobs_cv,
-                                    est_params=self._get_params('ml_m'))
+                                    est_params=self._get_params('ml_m'), method=self._predict_method['ml_m'])
         else:
             # several instruments: 2SLS
             m_hat = np.full((self._dml_data.n_obs, self._dml_data.n_instr), np.nan)
@@ -270,11 +288,12 @@ class DoubleMLPLIV(DoubleML):
             for i_instr in range(self._dml_data.n_instr):
                 x, this_z = check_X_y(x, z[:, i_instr])
                 m_hat[:, i_instr] = _dml_cv_predict(self._learner['ml_m'], x, this_z, smpls=smpls, n_jobs=n_jobs_cv,
-                                                    est_params=self._get_params('ml_m_' + self._dml_data.z_cols[i_instr]))
+                                                    est_params=self._get_params('ml_m_' + self._dml_data.z_cols[i_instr]),
+                                                    method=self._predict_method['ml_m'])
 
         # nuisance r
         r_hat = _dml_cv_predict(self._learner['ml_r'], x, d, smpls=smpls, n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_r'))
+                                est_params=self._get_params('ml_r'), method=self._predict_method['ml_r'])
 
         psi_a, psi_b = self._score_elements(y, z, d, g_hat, m_hat, r_hat, smpls)
 
@@ -305,8 +324,8 @@ class DoubleMLPLIV(DoubleML):
         else:
             assert callable(self.score)
             if self._dml_data.n_instr > 1:
-                raise NotImplementedError('callable score not implemented for DoubleMLPLIV.partialX '
-                                          'with several instruments')
+                raise NotImplementedError('Callable score not implemented for DoubleMLPLIV.partialX '
+                                          'with several instruments.')
             else:
                 assert self._dml_data.n_instr == 1
                 psi_a, psi_b = self.score(y, z, d,
@@ -321,7 +340,7 @@ class DoubleMLPLIV(DoubleML):
 
         # nuisance m
         r_hat = _dml_cv_predict(self._learner['ml_r'], xz, d, smpls=smpls, n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_r'))
+                                est_params=self._get_params('ml_r'), method=self._predict_method['ml_r'])
 
         if isinstance(self.score, str):
             assert self.score == 'partialling out'
@@ -329,7 +348,7 @@ class DoubleMLPLIV(DoubleML):
             psi_b = np.multiply(r_hat, y)
         else:
             assert callable(self.score)
-            raise NotImplementedError('callable score not implemented for DoubleMLPLIV.partialZ')
+            raise NotImplementedError('Callable score not implemented for DoubleMLPLIV.partialZ.')
 
         return psi_a, psi_b
 
@@ -341,15 +360,16 @@ class DoubleMLPLIV(DoubleML):
 
         # nuisance g
         g_hat = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls, n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_g'))
+                                est_params=self._get_params('ml_g'), method=self._predict_method['ml_g'])
 
         # nuisance m
         m_hat, m_hat_on_train = _dml_cv_predict(self._learner['ml_m'], xz, d, smpls=smpls, n_jobs=n_jobs_cv,
-                                                est_params=self._get_params('ml_m'), return_train_preds=True)
+                                                est_params=self._get_params('ml_m'), return_train_preds=True,
+                                                method=self._predict_method['ml_m'])
 
         # nuisance r
         m_hat_tilde = _dml_cv_predict(self._learner['ml_r'], x, m_hat_on_train, smpls=smpls, n_jobs=n_jobs_cv,
-                                      est_params=self._get_params('ml_r'))
+                                      est_params=self._get_params('ml_r'), method=self._predict_method['ml_r'])
 
         # compute residuals
         u_hat = y - g_hat
@@ -361,7 +381,7 @@ class DoubleMLPLIV(DoubleML):
             psi_b = np.multiply((m_hat-m_hat_tilde), u_hat)
         else:
             assert callable(self.score)
-            raise NotImplementedError('callable score not implemented for DoubleMLPLIV.partialXZ')
+            raise NotImplementedError('Callable score not implemented for DoubleMLPLIV.partialXZ.')
 
         return psi_a, psi_b
 
@@ -375,73 +395,32 @@ class DoubleMLPLIV(DoubleML):
                                'ml_m': None,
                                'ml_r': None}
 
-        g_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            g_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                g_grid_search = GridSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                             scoring=scoring_methods['ml_g'],
-                                             cv=g_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                g_grid_search = RandomizedSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                   scoring=scoring_methods['ml_g'],
-                                                   cv=g_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            g_tune_res.append(g_grid_search.fit(x[train_index, :], y[train_index]))
+        train_inds = [train_index for (train_index, _) in smpls]
+        g_tune_res = _dml_tune(y, x, train_inds,
+                               self._learner['ml_g'], param_grids['ml_g'], scoring_methods['ml_g'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
         if self._dml_data.n_instr > 1:
             # several instruments: 2SLS
             m_tune_res = {instr_var: list() for instr_var in self._dml_data.z_cols}
             z = self._dml_data.z
             for i_instr in range(self._dml_data.n_instr):
-                for idx, (train_index, test_index) in enumerate(smpls):
-                    x, this_z = check_X_y(x, z[:, i_instr])
-                    m_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-                    if search_mode == 'grid_search':
-                        m_grid_search = GridSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                     scoring=scoring_methods['ml_m'],
-                                                     cv=m_tune_resampling, n_jobs=n_jobs_cv)
-                    else:
-                        assert search_mode == 'randomized_search'
-                        m_grid_search = RandomizedSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                           scoring=scoring_methods['ml_m'],
-                                                           cv=m_tune_resampling, n_jobs=n_jobs_cv,
-                                                           n_iter=n_iter_randomized_search)
-                    m_tune_res[self._dml_data.z_cols[i_instr]].append(m_grid_search.fit(x[train_index, :],
-                                                                                        this_z[train_index]))
+                x, this_z = check_X_y(x, z[:, i_instr])
+                m_tune_res[self._dml_data.z_cols[i_instr]] = _dml_tune(this_z, x, train_inds,
+                                                                       self._learner['ml_m'], param_grids['ml_m'],
+                                                                       scoring_methods['ml_m'],
+                                                                       n_folds_tune, n_jobs_cv, search_mode,
+                                                                       n_iter_randomized_search)
         else:
             # one instrument: just identified
-            m_tune_res = list()
-            for idx, (train_index, test_index) in enumerate(smpls):
-                x, z = check_X_y(x, np.ravel(self._dml_data.z))
-                m_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-                if search_mode == 'grid_search':
-                    m_grid_search = GridSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                 scoring=scoring_methods['ml_m'],
-                                                 cv=m_tune_resampling, n_jobs=n_jobs_cv)
-                else:
-                    assert search_mode == 'randomized_search'
-                    m_grid_search = RandomizedSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                       scoring=scoring_methods['ml_m'],
-                                                       cv=m_tune_resampling, n_jobs=n_jobs_cv,
-                                                       n_iter=n_iter_randomized_search)
-                m_tune_res.append(m_grid_search.fit(x[train_index, :], z[train_index]))
+            x, z = check_X_y(x, np.ravel(self._dml_data.z))
+            m_tune_res = _dml_tune(z, x, train_inds,
+                                   self._learner['ml_m'], param_grids['ml_m'], scoring_methods['ml_m'],
+                                   n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
-        r_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            r_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                r_grid_search = GridSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                             scoring=scoring_methods['ml_r'],
-                                             cv=r_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                r_grid_search = RandomizedSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                                   scoring=scoring_methods['ml_r'],
-                                                   cv=r_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            r_tune_res.append(r_grid_search.fit(x[train_index, :], d[train_index]))
+        r_tune_res = _dml_tune(d, x, train_inds,
+                               self._learner['ml_r'], param_grids['ml_r'], scoring_methods['ml_r'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
         g_best_params = [xx.best_params_ for xx in g_tune_res]
         r_best_params = [xx.best_params_ for xx in r_tune_res]
@@ -473,20 +452,10 @@ class DoubleMLPLIV(DoubleML):
         if scoring_methods is None:
             scoring_methods = {'ml_r': None}
 
-        m_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            m_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                m_grid_search = GridSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                             scoring=scoring_methods['ml_r'],
-                                             cv=m_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                m_grid_search = RandomizedSearchCV(self._learner['ml_r'], param_grids['ml_r'],
-                                                   scoring=scoring_methods['ml_r'],
-                                                   cv=m_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            m_tune_res.append(m_grid_search.fit(xz[train_index, :], d[train_index]))
+        train_inds = [train_index for (train_index, _) in smpls]
+        m_tune_res = _dml_tune(d, xz, train_inds,
+                               self._learner['ml_r'], param_grids['ml_r'], scoring_methods['ml_r'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
         m_best_params = [xx.best_params_ for xx in m_tune_res]
 
@@ -511,38 +480,16 @@ class DoubleMLPLIV(DoubleML):
                                'ml_m': None,
                                'ml_r': None}
 
-        g_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            g_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                g_grid_search = GridSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                             scoring=scoring_methods['ml_g'],
-                                             cv=g_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                g_grid_search = RandomizedSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                   scoring=scoring_methods['ml_g'],
-                                                   cv=g_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            g_tune_res.append(g_grid_search.fit(x[train_index, :], y[train_index]))
-
-        m_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            m_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                m_grid_search = GridSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                             scoring=scoring_methods['ml_m'],
-                                             cv=m_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                m_grid_search = RandomizedSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                   scoring=scoring_methods['ml_m'],
-                                                   cv=m_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            m_tune_res.append(m_grid_search.fit(xz[train_index, :], d[train_index]))
+        train_inds = [train_index for (train_index, _) in smpls]
+        g_tune_res = _dml_tune(y, x, train_inds,
+                               self._learner['ml_g'], param_grids['ml_g'], scoring_methods['ml_g'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
+        m_tune_res = _dml_tune(d, xz, train_inds,
+                               self._learner['ml_m'], param_grids['ml_m'], scoring_methods['ml_m'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
         r_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
+        for idx, (train_index, _) in enumerate(smpls):
             m_hat = m_tune_res[idx].predict(xz[train_index, :])
             r_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
             if search_mode == 'grid_search':
