@@ -1,15 +1,13 @@
 import numpy as np
 from sklearn.utils import check_X_y
-from sklearn.model_selection import KFold
-from sklearn.model_selection import GridSearchCV, RandomizedSearchCV
+from sklearn.utils.multiclass import type_of_target
 
 from .double_ml import DoubleML
-from ._helper import _dml_cv_predict
+from ._helper import _dml_cv_predict, _dml_tune
 
 
 class DoubleMLPLR(DoubleML):
-    """
-    Double machine learning for partially linear regression models
+    """Double machine learning for partially linear regression models
 
     Parameters
     ----------
@@ -17,12 +15,15 @@ class DoubleMLPLR(DoubleML):
         The :class:`DoubleMLData` object providing the data and specifying the variables for the causal model.
 
     ml_g : estimator implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestRegressor`)
-        for the nuisance function :math:`g_0(X) = E[Y|X]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestRegressor`) for the nuisance function :math:`g_0(X) = E[Y|X]`.
 
     ml_m : estimator implementing ``fit()`` and ``predict()``
-        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g. :py:class:`sklearn.ensemble.RandomForestRegressor`)
-        for the nuisance function :math:`m_0(X) = E[D|X]`.
+        A machine learner implementing ``fit()`` and ``predict()`` methods (e.g.
+        :py:class:`sklearn.ensemble.RandomForestRegressor`) for the nuisance function :math:`m_0(X) = E[D|X]`.
+        For binary treatment variables :math:`D` (with values 0 and 1), a classifier implementing ``fit()`` and
+        ``predict_proba()`` can also be specified. If :py:func:`sklearn.base.is_classifier` returns ``True``,
+        ``predict_proba()`` is used otherwise ``predict()``.
 
     n_folds : int
         Number of folds.
@@ -68,7 +69,17 @@ class DoubleMLPLR(DoubleML):
 
     Notes
     -----
-    .. include:: ../../shared/models/plr.rst
+    **Partially linear regression (PLR)** models take the form
+
+    .. math::
+
+        Y = D \\theta_0 + g_0(X) + \\zeta, & &\\mathbb{E}(\\zeta | D,X) = 0,
+
+        D = m_0(X) + V, & &\\mathbb{E}(V | X) = 0,
+
+    where :math:`Y` is the outcome variable and :math:`D` is the policy variable of interest.
+    The high-dimensional vector :math:`X = (X_1, \\ldots, X_p)` consists of other confounding covariates,
+    and :math:`\\zeta` and :math:`V` are stochastic errors.
     """
     def __init__(self,
                  obj_dml_data,
@@ -87,8 +98,19 @@ class DoubleMLPLR(DoubleML):
                          dml_procedure,
                          draw_sample_splitting,
                          apply_cross_fitting)
-        self._learner = {'ml_g': self._check_learner(ml_g, 'ml_g'),
-                         'ml_m': self._check_learner(ml_m, 'ml_m')}
+
+        _ = self._check_learner(ml_g, 'ml_g', regressor=True, classifier=False)
+        ml_m_is_classifier = self._check_learner(ml_m, 'ml_m', regressor=True, classifier=True)
+        self._learner = {'ml_g': ml_g, 'ml_m': ml_m}
+        if ml_m_is_classifier:
+            if obj_dml_data.binary_treats.all():
+                self._predict_method = {'ml_g': 'predict', 'ml_m': 'predict_proba'}
+            else:
+                raise ValueError(f'The ml_m learner {str(ml_m)} was identified as classifier '
+                                 'but at least one treatment variable is not binary with values 0 and 1.')
+        else:
+            self._predict_method = {'ml_g': 'predict', 'ml_m': 'predict'}
+
         self._initialize_ml_nuisance_params()
 
     def _initialize_ml_nuisance_params(self):
@@ -98,33 +120,42 @@ class DoubleMLPLR(DoubleML):
         if isinstance(score, str):
             valid_score = ['IV-type', 'partialling out']
             if score not in valid_score:
-                raise ValueError('invalid score ' + score +
-                                 '\n valid score ' + ' or '.join(valid_score))
+                raise ValueError('Invalid score ' + score + '. ' +
+                                 'Valid score ' + ' or '.join(valid_score) + '.')
         else:
             if not callable(score):
-                raise ValueError('score should be either a string or a callable.'
-                                 ' %r was passed' % score)
+                raise TypeError('score should be either a string or a callable. '
+                                '%r was passed.' % score)
         return score
 
     def _check_data(self, obj_dml_data):
         if obj_dml_data.z_cols is not None:
-            raise ValueError('Incompatible data.\n'
+            raise ValueError('Incompatible data. ' +
                              ' and '.join(obj_dml_data.z_cols) +
-                             'have been set as instrumental variable(s).\n'
-                             'To fit an partially linear IV regression model use DoubleMLPLIV instead of DoubleMLPLR.')
+                             ' have been set as instrumental variable(s). '
+                             'To fit a partially linear IV regression model use DoubleMLPLIV instead of DoubleMLPLR.')
         return
 
     def _ml_nuisance_and_score_elements(self, smpls, n_jobs_cv):
         x, y = check_X_y(self._dml_data.x, self._dml_data.y)
         x, d = check_X_y(x, self._dml_data.d)
-        
+
         # nuisance g
         g_hat = _dml_cv_predict(self._learner['ml_g'], x, y, smpls=smpls, n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_g'))
-        
+                                est_params=self._get_params('ml_g'), method=self._predict_method['ml_g'])
+
         # nuisance m
         m_hat = _dml_cv_predict(self._learner['ml_m'], x, d, smpls=smpls, n_jobs=n_jobs_cv,
-                                est_params=self._get_params('ml_m'))
+                                est_params=self._get_params('ml_m'), method=self._predict_method['ml_m'])
+
+        if self._dml_data.binary_treats[self._dml_data.d_cols[self._i_treat]]:
+            binary_preds = (type_of_target(m_hat) == 'binary')
+            zero_one_preds = np.all((np.power(m_hat, 2) - m_hat) == 0)
+            if binary_preds & zero_one_preds:
+                raise ValueError(f'For the binary treatment variable {self._dml_data.d_cols[self._i_treat]}, '
+                                 f'predictions obtained with the ml_m learner {str(self._learner["ml_m"])} are also '
+                                 'observed to be binary with values 0 and 1. Make sure that for classifiers '
+                                 'probabilities and not labels are predicted.')
 
         psi_a, psi_b = self._score_elements(y, d, g_hat, m_hat, smpls)
 
@@ -158,35 +189,13 @@ class DoubleMLPLR(DoubleML):
             scoring_methods = {'ml_g': None,
                                'ml_m': None}
 
-        g_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            g_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                g_grid_search = GridSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                             scoring=scoring_methods['ml_g'],
-                                             cv=g_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                g_grid_search = RandomizedSearchCV(self._learner['ml_g'], param_grids['ml_g'],
-                                                   scoring=scoring_methods['ml_g'],
-                                                   cv=g_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            g_tune_res.append(g_grid_search.fit(x[train_index, :], y[train_index]))
-
-        m_tune_res = list()
-        for idx, (train_index, test_index) in enumerate(smpls):
-            m_tune_resampling = KFold(n_splits=n_folds_tune, shuffle=True)
-            if search_mode == 'grid_search':
-                m_grid_search = GridSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                             scoring=scoring_methods['ml_m'],
-                                             cv=m_tune_resampling, n_jobs=n_jobs_cv)
-            else:
-                assert search_mode == 'randomized_search'
-                m_grid_search = RandomizedSearchCV(self._learner['ml_m'], param_grids['ml_m'],
-                                                   scoring=scoring_methods['ml_m'],
-                                                   cv=m_tune_resampling, n_jobs=n_jobs_cv,
-                                                   n_iter=n_iter_randomized_search)
-            m_tune_res.append(m_grid_search.fit(x[train_index, :], d[train_index]))
+        train_inds = [train_index for (train_index, _) in smpls]
+        g_tune_res = _dml_tune(y, x, train_inds,
+                               self._learner['ml_g'], param_grids['ml_g'], scoring_methods['ml_g'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
+        m_tune_res = _dml_tune(d, x, train_inds,
+                               self._learner['ml_m'], param_grids['ml_m'], scoring_methods['ml_m'],
+                               n_folds_tune, n_jobs_cv, search_mode, n_iter_randomized_search)
 
         g_best_params = [xx.best_params_ for xx in g_tune_res]
         m_best_params = [xx.best_params_ for xx in m_tune_res]
